@@ -6,6 +6,8 @@
 
 #include <leatherman/util/environment.hpp>
 #include <leatherman/util/option_set.hpp>
+#include <boost/optional.hpp>
+#include <boost/filesystem.hpp>
 
 #include <string>
 #include <vector>
@@ -57,15 +59,33 @@ namespace leatherman { namespace execution {
          */
         preserve_arguments = (1 << 7),
         /**
-         * On Windows, create a new process group for the child process, but not a Job Object.
+         * Create a new process such that it can outlive its parent. This involves running it
+         * in a separate process group on Windows, and in a separate contract on Solaris.
          */
-        create_new_process_group = (1 << 8),
+        create_detached_process = (1 << 8),
         /**
          * Inherit locale environment variables from the current process. Limited to LC_ALL and
          * LOCALE, which are specifically overridden to "C" with merge_environment.
          * Will not override those variables if explicitly passed in an environment map.
          */
         inherit_locale = (1 << 9),
+        /**
+         * On windows, converts \r\n newlines to standard \n
+         */
+        convert_newlines = (1 << 10),
+        /**
+         * On POSIX systems, use `fork()` instead of `vfork()` when creating the new process.
+         * Ignored on windows.
+         * The `fork()` is typically slower than `vfork()` because it creates a copy
+         * of the parent's address space for the child process (`vfork()` lets the
+         * child process re-use the parent's address space) but safer from deadlocks
+         * if called from multi threaded processes.
+         */
+        thread_safe = (1 << 11),
+        /**
+         * Allow standard input to be unread, rather than failing if it is ignored.
+         */
+        allow_stdin_unread = (1 << 12),
         /**
          * A combination of all throw options.
          */
@@ -232,17 +252,19 @@ namespace leatherman { namespace execution {
      * Searches the given paths for the given executable file.
      * @param file The file to search for.
      * @param directories The directories to search.
+     * @param expand default true - expand first command to absolute path if founded in @param directories
      * @return Returns the full path or empty if the file could not be found.
      */
-    std::string which(std::string const& file, std::vector<std::string> const& directories = lth_util::environment::search_paths());
+    std::string which(std::string const& file, std::vector<std::string> const& directories = lth_util::environment::search_paths(), bool expand = true);
 
     /**
-     * Expands the executable in the command to the full path.
+     * Expands the executable in the command to the full path, expand is true, otherwise does not expand
      * @param command The command to expand.
      * @param directories The directories to search.
+     * @param expand default true - expand first command to absolute path if founded in @param directories
      * @return Returns the expanded command if the executable was found or empty if it was not found..
      */
-    std::string expand_command(std::string const& command, std::vector<std::string> const& directories = lth_util::environment::search_paths());
+    std::string expand_command(std::string const& command, std::vector<std::string> const& directories = lth_util::environment::search_paths(), bool expand = true);
 
     /**
      * Executes the given program.
@@ -341,7 +363,8 @@ namespace leatherman { namespace execution {
         lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment, execution_options::redirect_stderr_to_null });
 
     /**
-     * Executes the given program by writing the output of stdout and stderr to specified files.
+     * Executes the given program by writing the output of stdout and stderr to specified files. The output
+     * is processed line-by-line, so binary data isn't supported.
      * @param file The name or path of the program to execute.
      * @param arguments The arguments to pass to the program. On Windows they will be quoted as needed for spaces.
      * @param input A string to place on stdin for the child process before reading output.
@@ -350,7 +373,7 @@ namespace leatherman { namespace execution {
      * @param environment The environment variables to pass to the child process.
      * @param pid_callback The callback that is called with the pid of the child process. Defaults to no callback, in which case the pid will not be processed.
      * @param timeout The timeout, in seconds. Defaults to no timeout.
-     * @param options The execution options.  Defaults to trimming output and merging the environment.
+     * @param options The execution options. Defaults to trimming output and merging the environment.
      * @return Returns a result struct that will not contain the output of the streams for which a file was specified.
      *
      * Throws an execution_exception error in case it fails to open a file.
@@ -366,6 +389,36 @@ namespace leatherman { namespace execution {
         uint32_t timeout = 0,
         lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment });
 
+    /**
+     * Executes the given program by writing the output of stdout and stderr to specified files. The output
+     * is processed line-by-line, so binary data isn't supported.
+     * @param file The name or path of the program to execute.
+     * @param arguments The arguments to pass to the program. On Windows they will be quoted as needed for spaces.
+     * @param input A string to place on stdin for the child process before reading output.
+     * @param out_file The file where the output on stdout will be written.
+     * @param err_file The file where the output on stderr will be written.
+     * @param environment The environment variables to pass to the child process.
+     * @param pid_callback The callback that is called with the pid of the child process.
+     * @param timeout The timeout, in seconds.
+     * @param perms The file permissions to apply when creating the out_file and err_file.
+     *              On Windows this only toggles read-only.
+     * @param options The execution options. Defaults to trimming output and merging the environment.
+     * @return Returns a result struct that will not contain the output of the streams for which a file was specified.
+     *
+     * Throws an execution_exception error in case it fails to open a file.
+     */
+    result execute(
+        std::string const& file,
+        std::vector<std::string> const& arguments,
+        std::string const& input,
+        std::string const& out_file,
+        std::string const& err_file,
+        std::map<std::string, std::string> const& environment,
+        std::function<void(size_t)> pid_callback,
+        uint32_t timeout,
+        boost::optional<boost::filesystem::perms> perms,
+        lth_util::option_set<execution_options> const& options = { execution_options::trim_output, execution_options::merge_environment });
+
     /**
      * Executes the given program and returns each line of output.
      * @param file The name or path of the program to execute.
